/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.fidentis.analyst.procrustes.utils;

import cz.fidentis.analyst.feature.FeaturePoint;

import javax.vecmath.Vector3f;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

/**
 *
 * @author Jakub Kolman
 */
public class ProcrustesAnalysisUtils {

    /**
     * Sorts List by featurePoint type property
     *
     * @param featurePointList
     * @return ordered list of feature points by type
     */
    public static List<FeaturePoint> sortListByFeaturePointType(List<FeaturePoint> featurePointList) {
        Collections.sort(
                featurePointList, (FeaturePoint fp1, FeaturePoint fp2)
                -> fp1.getFeaturePointType().getType() - fp2.getFeaturePointType().getType() // Ascending
        );
        return featurePointList;
    }

    public static HashMap<Integer, FeaturePoint> generateFeaturePointHashMap(List<FeaturePoint> featurePointList) {
        HashMap<Integer, FeaturePoint> map = new HashMap<>();
        for (FeaturePoint fp : featurePointList) {
            if (fp.getFeaturePointType().getType() > 0) {
                map.put(fp.getFeaturePointType().getType(), fp);
            }
        }
        return map;
    }

    /**
     * Checks if two feature point lists have the same types of feature points.
     *
     * To use this method you need to supply ordered lists by Feature Point type
     * as parameters. Otherwise even if two lists contain the same featurepoints
     * it will return false.
     *
     * Use sort function {@link #sortByFeaturePointType} on feature point lists
     * two get correct results.
     *
     * @param featurePointList1
     * @param featurePointList2
     * @return true if two sorted lists by the feature point type contain the
     * same feature point types
     */
    public static boolean checkfeaturePointsType(List<FeaturePoint> featurePointList1, List<FeaturePoint> featurePointList2) {
        int i = 0;
        while (i < featurePointList1.size()) {
            if (featurePointList1.get(i).getFeaturePointType().getType() != featurePointList2.get(i).getFeaturePointType().getType()) {
                return false;
            }
        }
        return true;
    }

    /**
     * Creates array of distance values from feature point to origin (0,0,0).
     *
     * @param featurePointList
     * @return array of distances
     */
    public static double[] calculateMeanDistancesFromOrigin(List<FeaturePoint> featurePointList) {
        double[] distances = new double[featurePointList.size()];
        for (int i = 0; i < featurePointList.size(); i++) {
            distances[i] = calculatePointDistanceFromOrigin(featurePointList.get(i));
        }
        return distances;
    }

    /**
     * Finds centrioid from given feature point List
     *
     * @param featurePointList
     * @return centroid of feature points (Vector3F)
     */
    public static Vector3f findCenteroidOfFeaturePoints(List<FeaturePoint> featurePointList) {
        float x = 0;
        float y = 0;
        float z = 0;
        for (FeaturePoint fp : featurePointList) {
            x += fp.getX();
            y += fp.getY();
            z += fp.getZ();
        }
        return new Vector3f(x / featurePointList.size(), y / featurePointList.size(), z / featurePointList.size());
    }

//    /**
//     * Calculates distance of one feature point from another
//    
//     * @param fp1
//     * @param fp2
//     * @return 
//     */
//    public static double calculateDistanceOfTwoPoints(FeaturePoint fp1, FeaturePoint fp2) {
//        return Math.sqrt(
//                (Math.pow(fp1.getX(), 2) - Math.pow(fp2.getX(), 2))
//                + (Math.pow(fp1.getY(), 2) - Math.pow(fp2.getY(), 2))
//                + (Math.pow(fp1.getZ(), 2) - Math.pow(fp2.getZ(), 2)));
//    }
    /**
     * Calculates distance of single feature point from origin (0,0,0)
     *
     * @param fp
     * @return
     */
    public static double calculatePointDistanceFromOrigin(FeaturePoint fp) {
        return Math.sqrt(
                Math.pow(fp.getX(), 2)
                + Math.pow(fp.getY(), 2)
                + Math.pow(fp.getZ(), 2));
    }

}
